package org.eclipse.jdt.internal.compiler.apt.util;

package .tmp;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.net.URLClassLoader;
import java.nio.charset.Charset;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import java.util.Set;
import java.util.StringTokenizer;
import java.util.zip.ZipException;
import javax.tools.FileObject;
import javax.tools.JavaFileManager;
import javax.tools.JavaFileObject;
import javax.tools.StandardJavaFileManager;
import javax.tools.StandardLocation;
import org.eclipse.jdt.internal.compiler.apt.util.Archive;
import org.eclipse.jdt.internal.compiler.apt.util.EclipseFileObject;
import org.eclipse.jdt.internal.compiler.apt.util.Options;
import org.eclipse.jdt.internal.compiler.batch.FileSystem;
import org.eclipse.jdt.internal.compiler.batch.Main;
import org.eclipse.jdt.internal.compiler.env.AccessRule;
import org.eclipse.jdt.internal.compiler.env.AccessRuleSet;
import org.eclipse.jdt.internal.compiler.impl.CompilerOptions;
import org.eclipse.jdt.internal.compiler.util.Util;

public class EclipseFileManager implements StandardJavaFileManager {
  private static final String NO_EXTENSION = "";
  
  static final int HAS_EXT_DIRS = 1;
  
  static final int HAS_BOOTCLASSPATH = 2;
  
  static final int HAS_ENDORSED_DIRS = 4;
  
  static final int HAS_PROCESSORPATH = 8;
  
  Map<File, Archive> archivesCache;
  
  Charset charset;
  
  Locale locale;
  
  Map<String, Iterable<? extends File>> locations;
  
  int flags;
  
  public ResourceBundle bundle;
  
  public EclipseFileManager(Locale locale, Charset charset) {
    this.locale = (locale == null) ? Locale.getDefault() : locale;
    this.charset = (charset == null) ? Charset.defaultCharset() : charset;
    this.locations = new HashMap<>();
    this.archivesCache = new HashMap<>();
    try {
      setLocation(StandardLocation.PLATFORM_CLASS_PATH, getDefaultBootclasspath());
      Iterable<? extends File> defaultClasspath = getDefaultClasspath();
      setLocation(StandardLocation.CLASS_PATH, defaultClasspath);
      setLocation(StandardLocation.ANNOTATION_PROCESSOR_PATH, defaultClasspath);
    } catch (IOException iOException) {}
    try {
      this.bundle = Main.ResourceBundleFactory.getBundle(this.locale);
    } catch (MissingResourceException missingResourceException) {
      System.out.println("Missing resource : " + "org.eclipse.jdt.internal.compiler.batch.messages".replace('.', '/') + ".properties for locale " + locale);
    } 
  }
  
  public void close() throws IOException {
    if (this.locations != null)
      this.locations.clear(); 
    for (Archive archive : this.archivesCache.values())
      archive.close(); 
    this.archivesCache.clear();
  }
  
  private void collectAllMatchingFiles(File file, String normalizedPackageName, Set<JavaFileObject.Kind> kinds, boolean recurse, ArrayList<JavaFileObject> collector) {
    if (!isArchive(file)) {
      String path;
      File currentFile = new File(file, normalizedPackageName);
      if (!currentFile.exists())
        return; 
      try {
        path = currentFile.getCanonicalPath();
      } catch (IOException iOException) {
        return;
      } 
      if (File.separatorChar == '/') {
        if (!path.endsWith(normalizedPackageName))
          return; 
      } else if (!path.endsWith(normalizedPackageName.replace('/', File.separatorChar))) {
        return;
      } 
      File[] files = currentFile.listFiles();
      if (files != null) {
        byte b;
        int i;
        File[] arrayOfFile;
        for (i = (arrayOfFile = files).length, b = 0; b < i; ) {
          File f = arrayOfFile[b];
          if (f.isDirectory() && recurse) {
            collectAllMatchingFiles(file, String.valueOf(normalizedPackageName) + '/' + f.getName(), kinds, recurse, collector);
          } else {
            JavaFileObject.Kind kind = getKind(f);
            if (kinds.contains(kind))
              collector.add(new EclipseFileObject(String.valueOf(normalizedPackageName) + f.getName(), f.toURI(), kind, this.charset)); 
          } 
          b++;
        } 
      } 
    } else {
      String path;
      Archive archive = getArchive(file);
      if (archive == Archive.UNKNOWN_ARCHIVE)
        return; 
      String key = normalizedPackageName;
      if (!normalizedPackageName.endsWith("/"))
        path = String.valueOf(key) + '/'; 
      if (recurse) {
        for (String packageName : archive.allPackages()) {
          if (packageName.startsWith(path)) {
            List<String> types = archive.getTypes(packageName);
            if (types != null)
              for (String typeName : types) {
                JavaFileObject.Kind kind = getKind(getExtension(typeName));
                if (kinds.contains(kind))
                  collector.add(archive.getArchiveFileObject(String.valueOf(packageName) + typeName, this.charset)); 
              }  
          } 
        } 
      } else {
        List<String> types = archive.getTypes(path);
        if (types != null)
          for (String typeName : types) {
            JavaFileObject.Kind kind = getKind(getExtension(typeName));
            if (kinds.contains(kind))
              collector.add(archive.getArchiveFileObject(String.valueOf(path) + typeName, this.charset)); 
          }  
      } 
    } 
  }
  
  private Iterable<? extends File> concatFiles(Iterable<? extends File> iterable, Iterable<? extends File> iterable2) {
    ArrayList<File> list = new ArrayList<>();
    if (iterable2 == null)
      return iterable; 
    Iterator<? extends File> iterator;
    for (iterator = iterable.iterator(); iterator.hasNext();)
      list.add(iterator.next()); 
    for (iterator = iterable2.iterator(); iterator.hasNext();)
      list.add(iterator.next()); 
    return list;
  }
  
  public void flush() throws IOException {
    for (Archive archive : this.archivesCache.values())
      archive.flush(); 
  }
  
  private Archive getArchive(File f) {
    Archive archive = this.archivesCache.get(f);
    if (archive == null) {
      archive = Archive.UNKNOWN_ARCHIVE;
      if (f.exists()) {
        try {
          archive = new Archive(f);
        } catch (ZipException zipException) {
        
        } catch (IOException iOException) {}
        if (archive != null)
          this.archivesCache.put(f, archive); 
      } 
      this.archivesCache.put(f, archive);
    } 
    return archive;
  }
  
  public ClassLoader getClassLoader(JavaFileManager.Location location) {
    Iterable<? extends File> files = getLocation(location);
    if (files == null)
      return null; 
    ArrayList<URL> allURLs = new ArrayList<>();
    for (File f : files) {
      try {
        allURLs.add(f.toURI().toURL());
      } catch (MalformedURLException e) {
        throw new RuntimeException(e);
      } 
    } 
    URL[] result = new URL[allURLs.size()];
    return new URLClassLoader(allURLs.<URL>toArray(result), getClass().getClassLoader());
  }
  
  private Iterable<? extends File> getPathsFrom(String path) {
    ArrayList<FileSystem.Classpath> paths = new ArrayList<>();
    ArrayList<File> files = new ArrayList<>();
    try {
      processPathEntries(4, paths, path, this.charset.name(), false, false);
    } catch (IllegalArgumentException illegalArgumentException) {
      return null;
    } 
    for (FileSystem.Classpath classpath : paths)
      files.add(new File(classpath.getPath())); 
    return files;
  }
  
  Iterable<? extends File> getDefaultBootclasspath() {
    List<File> files = new ArrayList<>();
    String javaversion = System.getProperty("java.version");
    if (javaversion.length() > 3)
      javaversion = javaversion.substring(0, 3); 
    long jdkLevel = CompilerOptions.versionToJdkLevel(javaversion);
    if (jdkLevel < 3276800L)
      return null; 
    for (String fileName : Util.collectFilesNames())
      files.add(new File(fileName)); 
    return files;
  }
  
  Iterable<? extends File> getDefaultClasspath() {
    ArrayList<File> files = new ArrayList<>();
    String classProp = System.getProperty("java.class.path");
    if (classProp == null || classProp.length() == 0)
      return null; 
    StringTokenizer tokenizer = new StringTokenizer(classProp, File.pathSeparator);
    while (tokenizer.hasMoreTokens()) {
      String token = tokenizer.nextToken();
      File file = new File(token);
      if (file.exists())
        files.add(file); 
    } 
    return files;
  }
  
  private Iterable<? extends File> getEndorsedDirsFrom(String path) {
    ArrayList<FileSystem.Classpath> paths = new ArrayList<>();
    ArrayList<File> files = new ArrayList<>();
    try {
      processPathEntries(4, paths, path, this.charset.name(), false, false);
    } catch (IllegalArgumentException illegalArgumentException) {
      return null;
    } 
    for (FileSystem.Classpath classpath : paths)
      files.add(new File(classpath.getPath())); 
    return files;
  }
  
  private Iterable<? extends File> getExtdirsFrom(String path) {
    ArrayList<FileSystem.Classpath> paths = new ArrayList<>();
    ArrayList<File> files = new ArrayList<>();
    try {
      processPathEntries(4, paths, path, this.charset.name(), false, false);
    } catch (IllegalArgumentException illegalArgumentException) {
      return null;
    } 
    for (FileSystem.Classpath classpath : paths)
      files.add(new File(classpath.getPath())); 
    return files;
  }
  
  private String getExtension(File file) {
    String name = file.getName();
    return getExtension(name);
  }
  
  private String getExtension(String name) {
    int index = name.lastIndexOf('.');
    if (index == -1)
      return ""; 
    return name.substring(index);
  }
  
  public FileObject getFileForInput(JavaFileManager.Location location, String packageName, String relativeName) throws IOException {
    Iterable<? extends File> files = getLocation(location);
    if (files == null)
      throw new IllegalArgumentException("Unknown location : " + location); 
    String normalizedFileName = normalizedFileName(packageName, relativeName);
    for (File file : files) {
      if (file.isDirectory()) {
        File f = new File(file, normalizedFileName);
        if (f.exists())
          return (FileObject)new EclipseFileObject(String.valueOf(packageName) + File.separator + relativeName, f.toURI(), getKind(f), this.charset); 
        continue;
      } 
      if (isArchive(file)) {
        Archive archive = getArchive(file);
        if (archive != Archive.UNKNOWN_ARCHIVE && 
          archive.contains(normalizedFileName))
          return (FileObject)archive.getArchiveFileObject(normalizedFileName, this.charset); 
      } 
    } 
    return null;
  }
  
  private String normalizedFileName(String packageName, String relativeName) {
    StringBuilder sb = new StringBuilder();
    sb.append(normalized(packageName));
    if (sb.length() > 0)
      sb.append('/'); 
    sb.append(relativeName.replace('\\', '/'));
    return sb.toString();
  }
  
  public FileObject getFileForOutput(JavaFileManager.Location location, String packageName, String relativeName, FileObject sibling) throws IOException {
    Iterable<? extends File> files = getLocation(location);
    if (files == null)
      throw new IllegalArgumentException("Unknown location : " + location); 
    Iterator<? extends File> iterator = files.iterator();
    if (iterator.hasNext()) {
      File file = iterator.next();
      String normalizedFileName = String.valueOf(normalized(packageName)) + '/' + relativeName.replace('\\', '/');
      File f = new File(file, normalizedFileName);
      return (FileObject)new EclipseFileObject(String.valueOf(packageName) + File.separator + relativeName, f.toURI(), getKind(f), this.charset);
    } 
    throw new IllegalArgumentException("location is empty : " + location);
  }
  
  public JavaFileObject getJavaFileForInput(JavaFileManager.Location location, String className, JavaFileObject.Kind kind) throws IOException {
    if (kind != JavaFileObject.Kind.CLASS && kind != JavaFileObject.Kind.SOURCE)
      throw new IllegalArgumentException("Invalid kind : " + kind); 
    Iterable<? extends File> files = getLocation(location);
    if (files == null)
      throw new IllegalArgumentException("Unknown location : " + location); 
    String normalizedFileName = normalized(className);
    normalizedFileName = String.valueOf(normalizedFileName) + kind.extension;
    for (File file : files) {
      if (file.isDirectory()) {
        File f = new File(file, normalizedFileName);
        if (f.exists())
          return (JavaFileObject)new EclipseFileObject(className, f.toURI(), kind, this.charset); 
        continue;
      } 
      if (isArchive(file)) {
        Archive archive = getArchive(file);
        if (archive != Archive.UNKNOWN_ARCHIVE && 
          archive.contains(normalizedFileName))
          return (JavaFileObject)archive.getArchiveFileObject(normalizedFileName, this.charset); 
      } 
    } 
    return null;
  }
  
  public JavaFileObject getJavaFileForOutput(JavaFileManager.Location location, String className, JavaFileObject.Kind kind, FileObject sibling) throws IOException {
    if (kind != JavaFileObject.Kind.CLASS && kind != JavaFileObject.Kind.SOURCE)
      throw new IllegalArgumentException("Invalid kind : " + kind); 
    Iterable<? extends File> files = getLocation(location);
    if (files == null) {
      if (!location.equals(StandardLocation.CLASS_OUTPUT) && 
        !location.equals(StandardLocation.SOURCE_OUTPUT))
        throw new IllegalArgumentException("Unknown location : " + location); 
      if (sibling != null) {
        String str = normalized(className);
        int index = str.lastIndexOf('/');
        if (index != -1)
          str = str.substring(index + 1); 
        str = String.valueOf(str) + kind.extension;
        URI uri = sibling.toUri();
        URI uri2 = null;
        try {
          String path = uri.getPath();
          index = path.lastIndexOf('/');
          if (index != -1) {
            path = path.substring(0, index + 1);
            path = String.valueOf(path) + str;
          } 
          uri2 = new URI(uri.getScheme(), uri.getHost(), path, uri.getFragment());
        } catch (URISyntaxException uRISyntaxException) {
          throw new IllegalArgumentException("invalid sibling");
        } 
        return (JavaFileObject)new EclipseFileObject(className, uri2, kind, this.charset);
      } 
      String normalizedFileName = normalized(className);
      normalizedFileName = String.valueOf(normalizedFileName) + kind.extension;
      File f = new File(System.getProperty("user.dir"), normalizedFileName);
      return (JavaFileObject)new EclipseFileObject(className, f.toURI(), kind, this.charset);
    } 
    Iterator<? extends File> iterator = files.iterator();
    if (iterator.hasNext()) {
      File file = iterator.next();
      String normalizedFileName = normalized(className);
      normalizedFileName = String.valueOf(normalizedFileName) + kind.extension;
      File f = new File(file, normalizedFileName);
      return (JavaFileObject)new EclipseFileObject(className, f.toURI(), kind, this.charset);
    } 
    throw new IllegalArgumentException("location is empty : " + location);
  }
  
  public Iterable<? extends JavaFileObject> getJavaFileObjects(File... files) {
    return getJavaFileObjectsFromFiles(Arrays.asList(files));
  }
  
  public Iterable<? extends JavaFileObject> getJavaFileObjects(String... names) {
    return getJavaFileObjectsFromStrings(Arrays.asList(names));
  }
  
  public Iterable<? extends JavaFileObject> getJavaFileObjectsFromFiles(Iterable<? extends File> files) {
    ArrayList<JavaFileObject> javaFileArrayList = new ArrayList<>();
    for (File f : files) {
      if (f.isDirectory())
        throw new IllegalArgumentException("file : " + f.getAbsolutePath() + " is a directory"); 
      javaFileArrayList.add(new EclipseFileObject(f.getAbsolutePath(), f.toURI(), getKind(f), this.charset));
    } 
    return javaFileArrayList;
  }
  
  public Iterable<? extends JavaFileObject> getJavaFileObjectsFromStrings(Iterable<String> names) {
    ArrayList<File> files = new ArrayList<>();
    for (String name : names)
      files.add(new File(name)); 
    return getJavaFileObjectsFromFiles(files);
  }
  
  public JavaFileObject.Kind getKind(File f) {
    return getKind(getExtension(f));
  }
  
  private JavaFileObject.Kind getKind(String extension) {
    if (JavaFileObject.Kind.CLASS.extension.equals(extension))
      return JavaFileObject.Kind.CLASS; 
    if (JavaFileObject.Kind.SOURCE.extension.equals(extension))
      return JavaFileObject.Kind.SOURCE; 
    if (JavaFileObject.Kind.HTML.extension.equals(extension))
      return JavaFileObject.Kind.HTML; 
    return JavaFileObject.Kind.OTHER;
  }
  
  public Iterable<? extends File> getLocation(JavaFileManager.Location location) {
    if (this.locations == null)
      return null; 
    return this.locations.get(location.getName());
  }
  
  private Iterable<? extends File> getOutputDir(String string) {
    if ("none".equals(string))
      return null; 
    File file = new File(string);
    if (file.exists() && !file.isDirectory())
      throw new IllegalArgumentException("file : " + file.getAbsolutePath() + " is not a directory"); 
    ArrayList<File> list = new ArrayList<>(1);
    list.add(file);
    return list;
  }
  
  public boolean handleOption(String current, Iterator<String> remaining) {
    try {
      if ("-bootclasspath".equals(current)) {
        if (remaining.hasNext()) {
          Iterable<? extends File> bootclasspaths = getPathsFrom(remaining.next());
          if (bootclasspaths != null) {
            Iterable<? extends File> iterable = getLocation(StandardLocation.PLATFORM_CLASS_PATH);
            if ((this.flags & 0x4) == 0 && (
              this.flags & 0x1) == 0) {
              setLocation(StandardLocation.PLATFORM_CLASS_PATH, bootclasspaths);
            } else if ((this.flags & 0x4) != 0) {
              setLocation(StandardLocation.PLATFORM_CLASS_PATH, 
                  concatFiles(iterable, bootclasspaths));
            } else {
              setLocation(StandardLocation.PLATFORM_CLASS_PATH, 
                  prependFiles(iterable, bootclasspaths));
            } 
          } 
          this.flags |= 0x2;
          return true;
        } 
        throw new IllegalArgumentException();
      } 
      if ("-classpath".equals(current) || "-cp".equals(current)) {
        if (remaining.hasNext()) {
          Iterable<? extends File> classpaths = getPathsFrom(remaining.next());
          if (classpaths != null) {
            Iterable<? extends File> iterable = getLocation(StandardLocation.CLASS_PATH);
            if (iterable != null) {
              setLocation(StandardLocation.CLASS_PATH, 
                  concatFiles(iterable, classpaths));
            } else {
              setLocation(StandardLocation.CLASS_PATH, classpaths);
            } 
            if ((this.flags & 0x8) == 0)
              setLocation(StandardLocation.ANNOTATION_PROCESSOR_PATH, classpaths); 
          } 
          return true;
        } 
        throw new IllegalArgumentException();
      } 
      if ("-encoding".equals(current)) {
        if (remaining.hasNext()) {
          this.charset = Charset.forName(remaining.next());
          return true;
        } 
        throw new IllegalArgumentException();
      } 
      if ("-sourcepath".equals(current)) {
        if (remaining.hasNext()) {
          Iterable<? extends File> sourcepaths = getPathsFrom(remaining.next());
          if (sourcepaths != null)
            setLocation(StandardLocation.SOURCE_PATH, sourcepaths); 
          return true;
        } 
        throw new IllegalArgumentException();
      } 
      if ("-extdirs".equals(current)) {
        if (remaining.hasNext()) {
          Iterable<? extends File> iterable = getLocation(StandardLocation.PLATFORM_CLASS_PATH);
          setLocation(StandardLocation.PLATFORM_CLASS_PATH, 
              concatFiles(iterable, getExtdirsFrom(remaining.next())));
          this.flags |= 0x1;
          return true;
        } 
        throw new IllegalArgumentException();
      } 
      if ("-endorseddirs".equals(current)) {
        if (remaining.hasNext()) {
          Iterable<? extends File> iterable = getLocation(StandardLocation.PLATFORM_CLASS_PATH);
          setLocation(StandardLocation.PLATFORM_CLASS_PATH, 
              prependFiles(iterable, getEndorsedDirsFrom(remaining.next())));
          this.flags |= 0x4;
          return true;
        } 
        throw new IllegalArgumentException();
      } 
      if ("-d".equals(current)) {
        if (remaining.hasNext()) {
          Iterable<? extends File> outputDir = getOutputDir(remaining.next());
          if (outputDir != null)
            setLocation(StandardLocation.CLASS_OUTPUT, outputDir); 
          return true;
        } 
        throw new IllegalArgumentException();
      } 
      if ("-s".equals(current)) {
        if (remaining.hasNext()) {
          Iterable<? extends File> outputDir = getOutputDir(remaining.next());
          if (outputDir != null)
            setLocation(StandardLocation.SOURCE_OUTPUT, outputDir); 
          return true;
        } 
        throw new IllegalArgumentException();
      } 
      if ("-processorpath".equals(current)) {
        if (remaining.hasNext()) {
          Iterable<? extends File> processorpaths = getPathsFrom(remaining.next());
          if (processorpaths != null)
            setLocation(StandardLocation.ANNOTATION_PROCESSOR_PATH, processorpaths); 
          this.flags |= 0x8;
          return true;
        } 
        throw new IllegalArgumentException();
      } 
    } catch (IOException iOException) {}
    return false;
  }
  
  public boolean hasLocation(JavaFileManager.Location location) {
    return (this.locations != null && this.locations.containsKey(location.getName()));
  }
  
  public String inferBinaryName(JavaFileManager.Location location, JavaFileObject file) {
    String name = file.getName();
    JavaFileObject javaFileObject = null;
    int index = name.lastIndexOf('.');
    if (index != -1)
      name = name.substring(0, index); 
    try {
      javaFileObject = getJavaFileForInput(location, name, file.getKind());
    } catch (IOException iOException) {
    
    } catch (IllegalArgumentException illegalArgumentException) {
      return null;
    } 
    if (javaFileObject == null)
      return null; 
    return name.replace('/', '.');
  }
  
  private boolean isArchive(File f) {
    String extension = getExtension(f);
    return !(!extension.equalsIgnoreCase(".jar") && !extension.equalsIgnoreCase(".zip"));
  }
  
  public boolean isSameFile(FileObject fileObject1, FileObject fileObject2) {
    if (!(fileObject1 instanceof EclipseFileObject))
      throw new IllegalArgumentException("Unsupported file object class : " + fileObject1.getClass()); 
    if (!(fileObject2 instanceof EclipseFileObject))
      throw new IllegalArgumentException("Unsupported file object class : " + fileObject2.getClass()); 
    return fileObject1.equals(fileObject2);
  }
  
  public int isSupportedOption(String option) {
    return Options.processOptionsFileManager(option);
  }
  
  public Iterable<JavaFileObject> list(JavaFileManager.Location location, String packageName, Set<JavaFileObject.Kind> kinds, boolean recurse) throws IOException {
    Iterable<? extends File> allFilesInLocations = getLocation(location);
    if (allFilesInLocations == null)
      throw new IllegalArgumentException("Unknown location : " + location); 
    ArrayList<JavaFileObject> collector = new ArrayList<>();
    String normalizedPackageName = normalized(packageName);
    for (File file : allFilesInLocations)
      collectAllMatchingFiles(file, normalizedPackageName, kinds, recurse, collector); 
    return collector;
  }
  
  private String normalized(String className) {
    char[] classNameChars = className.toCharArray();
    for (int i = 0, max = classNameChars.length; i < max; i++) {
      switch (classNameChars[i]) {
        case '\\':
          classNameChars[i] = '/';
          break;
        case '.':
          classNameChars[i] = '/';
          break;
      } 
    } 
    return new String(classNameChars);
  }
  
  private Iterable<? extends File> prependFiles(Iterable<? extends File> iterable, Iterable<? extends File> iterable2) {
    if (iterable2 == null)
      return iterable; 
    ArrayList<File> list = new ArrayList<>();
    Iterator<? extends File> iterator;
    for (iterator = iterable2.iterator(); iterator.hasNext();)
      list.add(iterator.next()); 
    for (iterator = iterable.iterator(); iterator.hasNext();)
      list.add(iterator.next()); 
    return list;
  }
  
  public void setLocation(JavaFileManager.Location location, Iterable<? extends File> path) throws IOException {
    if (path != null) {
      if (location.isOutputLocation()) {
        int count = 0;
        for (Iterator<? extends File> iterator = path.iterator(); iterator.hasNext(); ) {
          iterator.next();
          count++;
        } 
        if (count != 1)
          throw new IllegalArgumentException("output location can only have one path"); 
      } 
      this.locations.put(location.getName(), path);
    } 
  }
  
  public void setLocale(Locale locale) {
    this.locale = (locale == null) ? Locale.getDefault() : locale;
    try {
      this.bundle = Main.ResourceBundleFactory.getBundle(this.locale);
    } catch (MissingResourceException e) {
      System.out.println("Missing resource : " + "org.eclipse.jdt.internal.compiler.batch.messages".replace('.', '/') + ".properties for locale " + locale);
      throw e;
    } 
  }
  
  public void processPathEntries(int defaultSize, ArrayList paths, String currentPath, String customEncoding, boolean isSourceOnly, boolean rejectDestinationPathOnJars) {
    String currentClasspathName = null;
    String currentDestinationPath = null;
    ArrayList<String> currentRuleSpecs = new ArrayList(defaultSize);
    StringTokenizer tokenizer = new StringTokenizer(currentPath, 
        String.valueOf(File.pathSeparator) + "[]", true);
    ArrayList<String> tokens = new ArrayList();
    while (tokenizer.hasMoreTokens())
      tokens.add(tokenizer.nextToken()); 
    int state = 0;
    String token = null;
    int cursor = 0, tokensNb = tokens.size(), bracket = -1;
    while (cursor < tokensNb && state != 99) {
      token = tokens.get(cursor++);
      if (token.equals(File.pathSeparator)) {
        switch (state) {
          case 0:
          case 3:
          case 10:
            break;
          case 1:
          case 2:
          case 8:
            state = 3;
            addNewEntry(paths, currentClasspathName, currentRuleSpecs, 
                customEncoding, currentDestinationPath, isSourceOnly, 
                rejectDestinationPathOnJars);
            currentRuleSpecs.clear();
            break;
          case 6:
            state = 4;
            break;
          case 7:
            throw new IllegalArgumentException(
                bind("configure.incorrectDestinationPathEntry", 
                  currentPath));
          case 11:
            cursor = bracket + 1;
            state = 5;
            break;
          default:
            state = 99;
            break;
        } 
      } else if (token.equals("[")) {
        switch (state) {
          case 0:
            currentClasspathName = "";
          case 1:
            bracket = cursor - 1;
          case 11:
            state = 10;
            break;
          case 2:
            state = 9;
            break;
          case 8:
            state = 5;
            break;
          default:
            state = 99;
            break;
        } 
      } else if (token.equals("]")) {
        switch (state) {
          case 6:
            state = 2;
            break;
          case 7:
            state = 8;
            break;
          case 10:
            state = 11;
            break;
          default:
            state = 99;
            break;
        } 
      } else {
        int i;
        switch (state) {
          case 0:
          case 3:
            state = 1;
            currentClasspathName = token;
            break;
          case 5:
            if (token.startsWith("-d ")) {
              if (currentDestinationPath != null)
                throw new IllegalArgumentException(
                    bind("configure.duplicateDestinationPathEntry", 
                      currentPath)); 
              currentDestinationPath = token.substring(3).trim();
              state = 7;
              break;
            } 
          case 4:
            if (currentDestinationPath != null)
              throw new IllegalArgumentException(
                  bind("configure.accessRuleAfterDestinationPath", 
                    currentPath)); 
            state = 6;
            currentRuleSpecs.add(token);
            break;
          case 9:
            if (!token.startsWith("-d ")) {
              state = 99;
              break;
            } 
            currentDestinationPath = token.substring(3).trim();
            state = 7;
            break;
          case 11:
            for (i = bracket; i < cursor; i++)
              currentClasspathName = String.valueOf(currentClasspathName) + (String)tokens.get(i); 
            state = 1;
            break;
          case 10:
            break;
          default:
            state = 99;
            break;
        } 
      } 
      if (state == 11 && cursor == tokensNb) {
        cursor = bracket + 1;
        state = 5;
      } 
    } 
    switch (state) {
      case 1:
      case 2:
      case 8:
        addNewEntry(paths, currentClasspathName, currentRuleSpecs, 
            customEncoding, currentDestinationPath, isSourceOnly, 
            rejectDestinationPathOnJars);
        break;
    } 
  }
  
  protected void addNewEntry(ArrayList<FileSystem.Classpath> paths, String currentClasspathName, ArrayList currentRuleSpecs, String customEncoding, String destPath, boolean isSourceOnly, boolean rejectDestinationPathOnJars) {
    int rulesSpecsSize = currentRuleSpecs.size();
    AccessRuleSet accessRuleSet = null;
    if (rulesSpecsSize != 0) {
      AccessRule[] accessRules = new AccessRule[currentRuleSpecs.size()];
      boolean rulesOK = true;
      Iterator<String> i = currentRuleSpecs.iterator();
      int j = 0;
      while (i.hasNext()) {
        String ruleSpec = i.next();
        char key = ruleSpec.charAt(0);
        String pattern = ruleSpec.substring(1);
        if (pattern.length() > 0) {
          switch (key) {
            case '+':
              accessRules[j++] = new AccessRule(pattern
                  .toCharArray(), 0);
              continue;
            case '~':
              accessRules[j++] = new AccessRule(pattern
                  .toCharArray(), 
                  16777496);
              continue;
            case '-':
              accessRules[j++] = new AccessRule(pattern
                  .toCharArray(), 
                  16777523);
              continue;
            case '?':
              accessRules[j++] = new AccessRule(pattern
                  .toCharArray(), 
                  16777523, true);
              continue;
          } 
          rulesOK = false;
          continue;
        } 
        rulesOK = false;
      } 
      if (rulesOK) {
        accessRuleSet = new AccessRuleSet(accessRules, (byte)0, currentClasspathName);
      } else {
        return;
      } 
    } 
    if ("none".equals(destPath))
      destPath = "none"; 
    if (rejectDestinationPathOnJars && destPath != null && (
      currentClasspathName.endsWith(".jar") || 
      currentClasspathName.endsWith(".zip")))
      throw new IllegalArgumentException(
          bind("configure.unexpectedDestinationPathEntryFile", 
            currentClasspathName)); 
    FileSystem.Classpath currentClasspath = FileSystem.getClasspath(
        currentClasspathName, 
        customEncoding, 
        isSourceOnly, 
        accessRuleSet, 
        destPath, null);
    if (currentClasspath != null)
      paths.add(currentClasspath); 
  }
  
  private String bind(String id, String binding) {
    return bind(id, new String[] { binding });
  }
  
  private String bind(String id, String[] arguments) {
    if (id == null)
      return "No message available"; 
    String message = null;
    try {
      message = this.bundle.getString(id);
    } catch (MissingResourceException missingResourceException) {
      return "Missing message: " + id + " in: " + "org.eclipse.jdt.internal.compiler.batch.messages";
    } 
    return MessageFormat.format(message, (Object[])arguments);
  }
}
